@@ -1127,7 +1127,30 @@ struct tile_feature {
 	size_t seq = 0;
 };
 
-static void add_mean(mvt_feature &feature, mvt_layer &layer, std::string const &accumulate_numeric) {
+static bool should_keep(std::string const &key,
+			std::set<std::string> const &keep,
+			std::set<std::string> const &exclude,
+			std::vector<std::string> const &exclude_prefix) {
+	if (keep.size() == 0 || keep.find(key) != keep.end()) {
+		if (exclude.find(key) != exclude.end()) {
+			return false;
+		}
+
+		for (auto const &prefix : exclude_prefix) {
+			if (starts_with(key, prefix)) {
+				return false;
+			}
+		}
+
+		return true;
+	}
+
+	return false;
+}
+
+static void add_mean(mvt_feature &feature, mvt_layer &layer, std::string const &accumulate_numeric,
+		     std::set<std::string> const &keep, std::set<std::string> const &exclude,
+		     std::vector<std::string> const &exclude_prefix) {
 	std::string accumulate_numeric_colon = accumulate_numeric + ":";
 
 	std::unordered_map<std::string, size_t> attributes;
@@ -1156,7 +1179,10 @@ static void add_mean(mvt_feature &feature, mvt_layer &layer, std::string const &
 				mvt_value mean;
 				mean.type = mvt_double;
 				mean.numeric_value.double_value = mvt_value_to_double(sum) / count_val;
-				layer.tag(feature, accumulate_numeric + ":mean:" + trunc, mean);
+
+				if (should_keep(key, keep, exclude, exclude_prefix)) {
+					layer.tag(feature, accumulate_numeric + ":mean:" + trunc, mean);
+				}
 			}
 		}
 	}
@@ -1170,7 +1196,9 @@ static void preserve_numeric(const std::string &key, const mvt_value &val,
 			     std::set<std::string> &keys,					   // key presence in the source feature
 			     std::map<std::string, size_t> &numeric_out_field,			   // key index in the output feature
 			     std::unordered_map<std::string, accum_state> &attribute_accum_state,  // accumulation state for preserve_attribute()
-			     key_pool &key_pool) {
+			     key_pool &key_pool,
+			     std::set<std::string> const &keep, std::set<std::string> const &exclude,
+			     std::vector<std::string> const &exclude_prefix) {
 	// If this is a numeric attribute, but there is also a prefix:sum (etc.) for the
 	// same attribute, we want to use that one instead of this one.
 
@@ -1203,6 +1231,10 @@ static void preserve_numeric(const std::string &key, const mvt_value &val,
 			// and then put it back on for the output field
 			std::string prefixed = accumulate_numeric + ":" + op.first + ":" + outkey;
 
+			if (!should_keep(prefixed, keep, exclude, exclude_prefix)) {
+				continue;
+			}
+
 			// Does it exist in the output feature already?
 
 			auto prefixed_attr = numeric_out_field.find(prefixed);
@@ -1263,27 +1295,6 @@ static void preserve_numeric(const std::string &key, const mvt_value &val,
 	}
 }
 
-static bool should_keep(std::string const &key,
-			std::set<std::string> const &keep,
-			std::set<std::string> const &exclude,
-			std::vector<std::string> const &exclude_prefix) {
-	if (keep.size() == 0 || keep.find(key) != keep.end()) {
-		if (exclude.find(key) != exclude.end()) {
-			return false;
-		}
-
-		for (auto const &prefix : exclude_prefix) {
-			if (starts_with(key, prefix)) {
-				return false;
-			}
-		}
-
-		return true;
-	}
-
-	return false;
-}
-
 static void handle_closepath_from_mvt(drawvec &geom) {
 	// mvt geometries close polygons with a mvt_closepath operation
 	// tippecanoe-internal geometries close polygons with a lineto to the initial point
@@ -1299,13 +1310,14 @@ static void handle_closepath_from_mvt(drawvec &geom) {
 	}
 }
 
-static void feature_out(std::vector<tile_feature> const &features, mvt_layer &outlayer,
+// returns true if a feature was output; false if it was clipped away
+static bool feature_out(std::vector<tile_feature> const &features, mvt_layer &outlayer,
 			std::set<std::string> const &keep,
 			std::set<std::string> const &exclude,
 			std::vector<std::string> const &exclude_prefix,
 			std::unordered_map<std::string, attribute_op> const &attribute_accum,
 			std::string const &accumulate_numeric,
-			key_pool &key_pool, int buffer) {
+			key_pool &key_pool, int buffer, bool include_nonaggregate) {
 	// Add geometry to output feature
 
 	drawvec geom = features[0].geom;
@@ -1356,22 +1368,22 @@ static void feature_out(std::vector<tile_feature> const &features, mvt_layer &ou
 			std::vector<mvt_value> full_values;
 			std::map<std::string, size_t> numeric_out_field;
 
-			for (size_t i = 0; i + 1 < features[0].tags.size(); i += 2) {
-				const std::string &key = features[0].layer->keys[features[0].tags[i]];
-				auto f = attribute_accum.find(key);
-				if (f != attribute_accum.end()) {
-					// this attribute has an accumulator, so convert it
-					full_keys.push_back(key_pool.pool(features[0].layer->keys[features[0].tags[i]]));
-					full_values.push_back(features[0].layer->values[features[0].tags[i + 1]]);
-				} else if (accumulate_numeric.size() > 0 && features[0].layer->values[features[0].tags[i + 1]].is_numeric()) {
-					// convert numeric for accumulation
-					numeric_out_field.emplace(key, full_keys.size());
-					full_keys.push_back(key_pool.pool(key));
-					full_values.push_back(features[0].layer->values[features[0].tags[i + 1]]);
-				} else {
-					// otherwise just tag it directly onto the output feature
-					if (should_keep(features[0].layer->keys[features[0].tags[i]], keep, exclude, exclude_prefix)) {
-						outlayer.tag(outfeature, features[0].layer->keys[features[0].tags[i]], features[0].layer->values[features[0].tags[i + 1]]);
+			auto const &f = features[0];
+			for (size_t i = 0; i + 1 < f.tags.size(); i += 2) {
+				const std::string &key = f.layer->keys[f.tags[i]];
+				if (should_keep(key, keep, exclude, exclude_prefix)) {
+					if (attribute_accum.find(key) != attribute_accum.end()) {
+						// this attribute has an accumulator, so convert it
+						full_keys.push_back(key_pool.pool(f.layer->keys[f.tags[i]]));
+						full_values.push_back(f.layer->values[f.tags[i + 1]]);
+					} else if (accumulate_numeric.size() > 0 && f.layer->values[f.tags[i + 1]].is_numeric()) {
+						// convert numeric for accumulation
+						numeric_out_field.emplace(key, full_keys.size());
+						full_keys.push_back(key_pool.pool(key));
+						full_values.push_back(f.layer->values[f.tags[i + 1]]);
+					} else if (include_nonaggregate) {
+						// otherwise just tag it directly onto the output feature
+						outlayer.tag(outfeature, f.layer->keys[f.tags[i]], f.layer->values[f.tags[i + 1]]);
 					}
 				}
 			}
@@ -1385,22 +1397,26 @@ static void feature_out(std::vector<tile_feature> const &features, mvt_layer &ou
 
 				for (size_t j = 0; j + 1 < features[i].tags.size(); j += 2) {
 					const std::string &key = features[i].layer->keys[features[i].tags[j]];
-					keys.insert(key);
+					if (should_keep(key, keep, exclude, exclude_prefix)) {
+						keys.insert(key);
+					}
 				}
 
 				for (size_t j = 0; j + 1 < features[i].tags.size(); j += 2) {
 					const std::string &key = features[i].layer->keys[features[i].tags[j]];
-
-					auto f = attribute_accum.find(key);
-					if (f != attribute_accum.end()) {
-						mvt_value val = features[i].layer->values[features[i].tags[j + 1]];
-						preserve_attribute(f->second, key, val, full_keys, full_values, attribute_accum_state, key_pool);
-					} else if (accumulate_numeric.size() > 0) {
-						const mvt_value &val = features[i].layer->values[features[i].tags[j + 1]];
-						if (val.is_numeric()) {
-							preserve_numeric(key, val, full_keys, full_values,
-									 accumulate_numeric,
-									 keys, numeric_out_field, attribute_accum_state, key_pool);
+					if (should_keep(key, keep, exclude, exclude_prefix)) {
+						auto found = attribute_accum.find(key);
+						if (found != attribute_accum.end()) {
+							mvt_value val = features[i].layer->values[features[i].tags[j + 1]];
+							preserve_attribute(found->second, key, val, full_keys, full_values, attribute_accum_state, key_pool);
+						} else if (accumulate_numeric.size() > 0) {
+							const mvt_value &val = features[i].layer->values[features[i].tags[j + 1]];
+							if (val.is_numeric()) {
+								preserve_numeric(key, val, full_keys, full_values,
+										 accumulate_numeric,
+										 keys, numeric_out_field, attribute_accum_state, key_pool,
+										 keep, exclude, exclude_prefix);
+							}
 						}
 					}
 				}
@@ -1416,9 +1432,9 @@ static void feature_out(std::vector<tile_feature> const &features, mvt_layer &ou
 			}
 
 			if (accumulate_numeric.size() > 0) {
-				add_mean(outfeature, outlayer, accumulate_numeric);
+				add_mean(outfeature, outlayer, accumulate_numeric, keep, exclude, exclude_prefix);
 			}
-		} else {
+		} else if (include_nonaggregate) {
 			for (size_t i = 0; i + 1 < features[0].tags.size(); i += 2) {
 				if (should_keep(features[0].layer->keys[features[0].tags[i]], keep, exclude, exclude_prefix)) {
 					outlayer.tag(outfeature, features[0].layer->keys[features[0].tags[i]], features[0].layer->values[features[0].tags[i + 1]]);
@@ -1427,7 +1443,10 @@ static void feature_out(std::vector<tile_feature> const &features, mvt_layer &ou
 		}
 
 		outlayer.features.push_back(std::move(outfeature));
+		return true;
 	}
+
+	return false;
 }
 
 static struct preservecmp {
@@ -1719,21 +1738,24 @@ mvt_tile assign_to_bins(mvt_tile &features,
 
 	for (size_t i = 0; i < outfeatures.size(); i++) {
 		if (outfeatures[i].size() > 1) {
-			feature_out(outfeatures[i], outlayer,
-				    keep, exclude, exclude_prefix, attribute_accum,
-				    accumulate_numeric, key_pool, buffer);
-			mvt_feature &nfeature = outlayer.features.back();
-			mvt_value val;
-			val.type = mvt_uint;
-			val.numeric_value.uint_value = outfeatures[i].size() - 1;
+			if (feature_out(outfeatures[i], outlayer,
+					keep, exclude, exclude_prefix, attribute_accum,
+					accumulate_numeric, key_pool, buffer, true)) {
+				mvt_feature &nfeature = outlayer.features.back();
+				mvt_value val;
+				val.type = mvt_uint;
+				val.numeric_value.uint_value = outfeatures[i].size() - 1;
 
-			std::string attrname;
-			if (accumulate_numeric.size() == 0) {
-				attrname = "tippecanoe:count";
-			} else {
-				attrname = accumulate_numeric + ":count";
+				std::string attrname;
+				if (accumulate_numeric.size() == 0) {
+					attrname = "tippecanoe:count";
+				} else {
+					attrname = accumulate_numeric + ":count";
+				}
+				if (should_keep(attrname, keep, exclude, exclude_prefix)) {
+					outlayer.tag(nfeature, attrname, val);
+				}
 			}
-			outlayer.tag(nfeature, attrname, val);
 		}
 	}
 
@@ -1881,7 +1903,7 @@ std::string overzoom(std::vector<source_tile> const &tiles, int nz, int nx, int
 
 				if (flush_multiplier_cluster) {
 					if (pending_tile_features.size() > 0) {
-						feature_out(pending_tile_features, *outlayer, keep, exclude, exclude_prefix, attribute_accum, accumulate_numeric, key_pool, -1);
+						feature_out(pending_tile_features, *outlayer, keep, exclude, exclude_prefix, attribute_accum, accumulate_numeric, key_pool, -1, bins.size() == 0);
 						if (outlayer->features.size() >= feature_limit) {
 							break;
 						}
@@ -1941,7 +1963,7 @@ std::string overzoom(std::vector<source_tile> const &tiles, int nz, int nx, int
 			}
 
 			if (pending_tile_features.size() > 0) {
-				feature_out(pending_tile_features, *outlayer, keep, exclude, exclude_prefix, attribute_accum, accumulate_numeric, key_pool, -1);
+				feature_out(pending_tile_features, *outlayer, keep, exclude, exclude_prefix, attribute_accum, accumulate_numeric, key_pool, -1, bins.size() == 0);
 				pending_tile_features.clear();
 				if (outlayer->features.size() >= feature_limit) {
 					break;
